#    most of this task is now taken from the standard application

set VERSION "TUDA"

#    put some default environment variables before loading common 

if {[info exists env(RDOUThdwre)] == 0} then {set env(RDOUThdwre) VME} 

loadsource sys.common
loadsource GroupDefn
loadsource fltohex
loadsource epicsDefn

set ScalerType v560


set frametitle "MIDAS Data Acquisition Experiment Control (TUDA)"

set VMEPresent 1
set VMEEnable 1

set TSEnable 1

set histEnable 1
set HistMode 1

set txEnable 1

set TestBuffer ""

lappend actionlist [list "Show histogram name translations" "Show_Xlate"]
lappend actionlist [list "Edit histogram translation file"  "edit_xlate"]


set Do_Ancillary 0


proc create_spectra {} {
   global SpectrumServer HistMode EGreport

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set range 2048
   
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat"

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

#   for silena 9418

    create_S9418_spectra
   
#    for CAEN V767

    create_V767_spectra

#    for CAEN V890

    create_V890_spectra
   
   set EGreport $EGreport_SAVE

   clear-footer
}

proc define_VME_addresses {} {

#    load hardware configuration into readout software

     ClearConfig

#    the S9418 VME modules

     define_S9418_config

#    the V767 VME modules

     define_V767_config

#    the V890 VME modules

     define_V890_config

#    then the acquisition control module

     define_sac_config
}

proc click-vme {action} {

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
    }

   set-busy
 
   set-footer "sending $p to VME control task"
   set z [catch {send vmecontrol $p} m]

   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   if {$action == 3} then {
       define_VME_addresses
       do_V767_setup
       do_V890_setup
   }

   redisplay-vme
}

proc redisplay-vme {} {
    global VMEEnable vmestate

  if {$VMEEnable == 0} then {return}

   set vmestate [expr [read-register sac#1.Enable] & 1]
   :modify VME :value $vmestate
}

proc Stop_Electronics {} {

       write-register sac#1.STR 0
    
       Do_Scalers
       Do_EPICS
       
}

proc Go_Electronics {} {

    write-register sac#1.MON0 2
    write-register sac#1.STR 9
    write-register sac#1.ARM 0
}

proc Reset_Electronics {} {write-register sac#1.EVCLR 0}

proc Check-DA {} {
   global OS RDOUThdwre Alive Do_Ancillary

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

#    this check only works if GOING

   set state [read-register $RDOUThdwre.Sstate]
   if {$state != 2} then {return}
   
   after 10000 Check-DA

   set-footer "Checking Data Acqusition Program"

   set Alive [read-register $RDOUThdwre.CAlive]
   write-register $RDOUThdwre.CAlive 0

   if {$Alive == 0} then {
#        midas-warning "Warning - Check the Data Acquisition Program - It may have crashed"
         set-footer "Warning - No data being received or the DA Program has crashed"
   } else {
         set-footer "Data Acquisition running OK"
   }
   
   click-redisplay
   
   if {$Do_Ancillary >= 0} then {incr Do_Ancillary}
   if {$Do_Ancillary == 6} {

       Do_Scalers
       Do_EPICS
       set-footer "[date] Generated scaler and EPICS data"
       set Do_Ancillary 0
   }

}


proc Do_Scalers {} {
    global ScalerType Scaler_Base  EGreport
    
    set EGreport_SAVE $EGreport
    set EGreport -1

     inform $ScalerType display_counters
     set maxcounter [inform $ScalerType inquire maxcounter]

     if {$maxcounter == 0} {return}

     execute-command 10 "Start Scalers Event" "" 2

     for {set i 0} {$i < $maxcounter} {incr i} {
         set scaler [inform $ScalerType inquire data $i]
         set datah [format "%04x" [expr ($scaler >> 16) & 0x0000ffff]]
         set labelh [format "%04x" [expr (($i*2)*256) + $Scaler_Base]]

         set datal [format "%04x" [expr $scaler & 0x0000ffff]]
         set labell [format "%04x" [expr ((($i*2)+1)*256) + $Scaler_Base]]

        execute-command 11 "Define Scalers Event" "$labelh$datah $labell$datal" 2
     }

     execute-command 12 "Complete Scalers Event" "" 2
     
     set EGreport $EGreport_SAVE

}

proc Do_EPICS {} {
    global env EPICS_Base EPICS EGreport

    if {![info exists env(EPICSpresent)]} then {return}
    
#    scan though the EPICS parameter table and generate the command for caget
#    also extract the item base and data mode for use later

    set cmd "exec caget"
    set i 0
    foreach item $EPICS {
        append cmd " [first $item]"
        set id($i) [second $item]
        set mode($i) [third $item]
        incr i
    }
    
#    now execute the caget command 
    
    set z [catch {eval $cmd} m]
    
    if {$z != 0} then {
        set-footer "Warning EPICS channel access failed - $m"
    } else {
    
       set EGreport_SAVE $EGreport
       set EGreport -1
       
#    start generation of the EPICS data event
    
       execute-command 10 "Start EPICS Event" "" 2
       
#    for each line in the response generate a data word
         
       set r [split $m \n]
       set i 0
       foreach item $r {
       
         set vl [second $item]
         
         switch $mode($i) {
         
         0   {
         
#    32 bit floating data

                 set xvl [float2hex $vl]
         
                 set datah  [string range $xvl 2 5]
                 set labelh [format "%04x" [expr ($id($i)*256) + $EPICS_Base]]
         
                 set datal  [string range $xvl 6 9]
                 set labell [format "%04x" [expr (($id($i)+1)*256) + $EPICS_Base]]
         
                 execute-command 11 "Define EPICS Event" "$labelh$datah $labell$datal" 2
              }
              
         1    {
         
#    32 bit integer data

                 set datah  [format "%04x" [expr ($vl >> 16) & 0x0000ffff]]
                 set labelh [format "%04x" [expr ($id($i)*256) + $EPICS_Base]]
         
                 set datal  [format "%04x" [expr $vl & 0x0000ffff]]
                 set labell [format "%04x" [expr (($id($i)+1)*256) + $EPICS_Base]]
         
                 execute-command 11 "Define EPICS Event" "$labelh$datah $labell$datal" 2
              }
              
         2   {
         
#    16 bit integer data

                 set datal  [format "%04x" [expr $vl & 0x0000ffff]]
                 set labell [format "%04x" [expr ($id($i)*256) + $EPICS_Base]]
         
                 execute-command 11 "Define EPICS Event" "$labell$datal" 2
              }
          }
          incr i
       }
       
#    complete the EPICS data event

       execute-command 12 "Complete EPICS Event" "" 2
       
       set EGreport $EGreport_SAVE
    }

}
